#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

import itk
from sys import argv


#
#  Check input parameters
#  INPUTS(fixedImage):  {BrainProtonDensitySliceBorder20.png}
#  INPUTS(movingImage): {BrainProtonDensitySliceShifted13x17y.png}
#
if len(argv) < 4:
    print 'Missing Parameters'
    print 'Usage: ImageRegistration4.py fixedImageFile  movingImageFile outputImagefile'
    exit()


#
#  Define data types
#
FixedImageType   = itk.Image[itk.F, 2]
MovingImageType  = itk.Image[itk.F, 2]
TransformType    = itk.TranslationTransform[itk.D, 2]
OptimizerType    = itk.RegularStepGradientDescentOptimizerv4[itk.D]
RegistrationType = itk.ImageRegistrationMethodv4[FixedImageType,
                                                 MovingImageType]
MetricType       = itk.MattesMutualInformationImageToImageMetricv4[FixedImageType,
                                                                   MovingImageType]


#
#  Read the fixed and moving images using filenames
#  from the command line arguments
#
fixedImageReader  = itk.ImageFileReader[FixedImageType].New()
movingImageReader = itk.ImageFileReader[MovingImageType].New()

fixedImageReader.SetFileName(  argv[1])
movingImageReader.SetFileName( argv[2])

fixedImageReader.Update()
movingImageReader.Update()

fixedImage  = fixedImageReader.GetOutput()
movingImage = movingImageReader.GetOutput()


#
#  Instantiate the classes for the registration framework
#
registration = RegistrationType.New()
imageMetric  = MetricType.New()
transform    = TransformType.New()
optimizer    = OptimizerType.New()

registration.SetOptimizer(optimizer)
registration.SetMetric(imageMetric)

numberOfBins = 24

imageMetric.SetNumberOfHistogramBins( numberOfBins );
imageMetric.SetUseMovingImageGradientFilter( False );
imageMetric.SetUseFixedImageGradientFilter( False );

registration.SetFixedImage(fixedImage)
registration.SetMovingImage(movingImage)

registration.SetInitialTransform(transform)


#
#  Define optimizer parameters
#
optimizer.SetLearningRate(8.00)
optimizer.SetMinimumStepLength(0.001)
optimizer.SetNumberOfIterations(100)
optimizer.ReturnBestParametersAndValueOn();
optimizer.SetRelaxationFactor(0.8)


#
# One level registration process without shrinking and smoothing.
#
registration.SetNumberOfLevels(1)
registration.SetSmoothingSigmasPerLevel([0])
registration.SetShrinkFactorsPerLevel([1])

registration.SetMetricSamplingStrategy( RegistrationType.RANDOM );
registration.SetMetricSamplingPercentage( 0.20 );


#
# Iteration Observer
#
def iterationUpdate():
    currentParameter = registration.GetOutput().Get().GetParameters()
    print "M: %f   P: %f %f " % ( optimizer.GetValue(),
                                  currentParameter.GetElement(0),
                                  currentParameter.GetElement(1))

iterationCommand = itk.PyCommand.New()
iterationCommand.SetCommandCallable(iterationUpdate)
optimizer.AddObserver(itk.IterationEvent(),iterationCommand)

print "Starting registration"


#
#  Start the registration process
#
registration.Update()


#
# Get the final parameters of the transformation
#
finalParameters = registration.GetOutput().Get().GetParameters()

print "Final Registration Parameters "
print "Translation X =  %f" % (finalParameters.GetElement(0),)
print "Translation Y =  %f" % (finalParameters.GetElement(1),)


#
# Now, we use the final transform for resampling the
# moving image.
#
resampler = itk.ResampleImageFilter[MovingImageType,FixedImageType].New()
resampler.SetTransform(registration.GetTransform())
resampler.SetInput(movingImageReader.GetOutput())

region = fixedImage.GetLargestPossibleRegion()

resampler.SetSize(region.GetSize())
resampler.SetOutputOrigin(fixedImage.GetOrigin())
resampler.SetOutputSpacing(fixedImage.GetSpacing())
resampler.SetOutputDirection(fixedImage.GetDirection())
resampler.SetDefaultPixelValue(100)

OutputImageType  = itk.Image[itk.UC, 2]
outputCast = itk.CastImageFilter[FixedImageType, OutputImageType].New()
outputCast.SetInput(resampler.GetOutput())


#
#  Write the resampled image
#
writer = itk.ImageFileWriter[OutputImageType].New()
writer.SetFileName( argv[3] )
writer.SetInput( outputCast.GetOutput() )
writer.Update()



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
#  Example on the use of the MeanImageFilter with function calls
#

import itk
from sys import argv

image = itk.imread(argv[1])
filtered_image = itk.MeanImageFilter(image, Radius=int(argv[3]))
itk.imwrite(filtered_image, argv[2])



#!/usr/bin/python
#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
# \author Hans J. Johnson
#
# This script is designed to help change the copyright notices in all ITK files to a common format.
# For files that are .h, .cxx, .hxx, .c, if there is no other copyright information, add the itkCopyright.

from __future__ import print_function

import re
import sys
import os

## New license as specified on: https://itk.org/Wiki/ITK_Release_4/Licensing
NewITKCopyrightNotice="""/*=========================================================================
 *
 *  Copyright Insight Software Consortium
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *=========================================================================*/
"""
NewVTKDependantCopyrightNotice="""/*=========================================================================
 *
 *  Portions of this file are subject to the VTK Toolkit Version 3 copyright.
 *
 *  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
 *
 *  For complete copyright, license and disclaimer of warranty information
 *  please refer to the NOTICE file at the top of the ITK source tree.
 *
 *=========================================================================*/
"""

## Patterns that match the old copyright notice sections
## ITK only copyright
ITKOnlyOldHeader=""" */\* *==.*Program:.*Insight Segmentation & Registration Toolkit.*Copyright .* Insight.*Consortium. All rights reserved.*See ITKCopyright.txt or https://www.itk.org/HTML/Copyright.htm for details.[\n\r ]*This software is distributed WITHOUT ANY WARRANTY; without even.*the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR.*PURPOSE.  See the above copyright notices for more information.*=== *\*/[\n\r ]*"""
ITKOnlyOldRE=re.compile(ITKOnlyOldHeader,re.MULTILINE|re.DOTALL|re.IGNORECASE)

## Files that originated in VTK, and now have ITK also
ITKVTKOldHeader=""" */\* *==.*Program:.*Insight Segmentation & Registration Toolkit.*Copyright .* Insight Software Consortium. All rights reserved.*See ITKCopyright.txt or https://www.itk.org/HTML/Copyright.htm for details.[\n\r ]*.*VTKCopyright.txt.*This software is distributed WITHOUT ANY WARRANTY; without even.*the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR.*PURPOSE.  See the above copyright notices for more information.*=== *\*/[\n\r ]*"""
ITKVTKOldRE=re.compile(ITKVTKOldHeader,re.MULTILINE|re.DOTALL|re.IGNORECASE)

## Looking for new files.
NewITKHeader=""" */\* *==.*http://www.apache.org/licenses/LICENSE-2.0.txt.*=== *\*/"""
NewITKHeaderRE=re.compile(NewITKHeader,re.MULTILINE|re.DOTALL|re.IGNORECASE)

eolSpaceRemove=re.compile(r'  *$',re.MULTILINE)

## The exception list contains files that should not have the ITK copyright notices added.
ExclusionList=['Utilities','.git']

ExtensionsThatNeedCopyright=['.cxx','.c','.h','.hxx']


############
############
############
############
############
############
############

if len(sys.argv) != 2:
    print("USAGE:  {0} <Top of ITK tree to process>".format(sys.argv[0]))
    sys.exit(-1)

HeadOfITKTree=sys.argv[1]

for top,directory,files in os.walk(HeadOfITKTree):
    ## First remove Excluded directories
    for dd in directory:
      if dd[0] == '.': #Skip all directories that begin with '.'
        directory.remove(dd)
        continue
      if dd in ExclusionList:
          directory.remove(dd)
          continue
    ##  Now process each file
    for ff in files:
      if ff in ExclusionList:
        files.remove(ff)
        continue
      if ff[0] == '.': #Skip all files that begin with '.'
        files.remove(ff)
        #print("@@@@@@@",ff)
        continue
      currFile=os.path.join(top,ff)
      print(currFile)

      infile=open(currFile,'r')
      file_text=infile.read()
      newstring=file_text # default output to input, just in case all search patterns fail
      infile.close()

      substitutionMade=0
      testITKOnlySearch=ITKOnlyOldRE.search(file_text)
      if testITKOnlySearch:
        print("{0} is ITKOnlyHeader".format(currFile))
        newstring=ITKOnlyOldRE.sub(NewITKCopyrightNotice,file_text)
        newstring=eolSpaceRemove.sub("",newstring) ## a few files still have eol spaces
        substitutionMade=1

      testITKVTKSearch=ITKVTKOldRE.search(file_text)
      if testITKVTKSearch:
        print("{0} is VTKITKHeader".format(currFile))
        newstring=ITKVTKOldRE.sub(NewITKCopyrightNotice+NewVTKDependantCopyrightNotice,file_text)
        newstring=eolSpaceRemove.sub("",newstring) ## a few files still have eol spaces
        substitutionMade=1

      ##Add new copyright if it had not already existed.
      root,ext=os.path.splitext(currFile)
      if ext in ExtensionsThatNeedCopyright:
        testNewITKHeaderRE=NewITKHeaderRE.search(file_text) # see if new CopyRight notice already exists.
        if testNewITKHeaderRE:
          print("Already Processed {0}".format(currFile))
        elif (substitutionMade == 0):
          print("{0} needed copyright header.".format(currFile))
          newstring=NewITKCopyrightNotice+file_text
          newstring=eolSpaceRemove.sub("",newstring) ## a few files still have eol spaces

      outfile=open(currFile,'w')
      outfile.write(newstring)
      outfile.close()



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

"""itk: Top-level container package for ITK wrappers."""
import itkBase
import itkConfig
import itkLazy
import itkTypes
import itkExtras
import os
import sys

thisModule = sys.modules[__name__]

def _GetLazyAttributes(lazyAttributes):
    templateNames = [t[0] for t in data['templates']]
    isInLibrary = [t[3] for t in data['templates'] if len(t) > 3 ]
    attributes = dict([(n, module) for n in templateNames])
    attributesInModule = dict([(n, belongs) for n,belongs in zip(templateNames,isInLibrary)])
    items = attributes.items()
    for k, v in items:
        if isInLibrary and attributesInModule[k] is True:
            lazyAttributes.setdefault(k,[]).insert(0,v)
        else:
            lazyAttributes.setdefault(k,[]).append(v)

if itkConfig.LazyLoading:
    # If we are loading lazily (on-demand), make a dict mapping the available
    # classes/functions/etc. (read from the configuration modules) to the
    # modules they are declared in. Then pass that dict to a LazyITKModule
    # instance and (later) do some surgery on sys.modules so that the 'itk'
    # module becomes that new instance instead of what is executed from this
    # file.
    lazyAttributes = {}
    for module, data in itkBase.module_data.items():
        _GetLazyAttributes(lazyAttributes)

    if isinstance(thisModule, itkLazy.LazyITKModule):
        # Handle reload case where we've already done this once.
        # If we made a new module every time, multiple reload()s would fail
        # because the identity of sys.modules['itk'] would always be changing.
        thisModule.__init__(__name__, lazyAttributes)
        del lazyAttributes
    else:
        thisModule = itkLazy.LazyITKModule(__name__, lazyAttributes)
else:
    # We're not lazy-loading. Just load the modules in the order specified in
    # the known_modules list for consistency.
    for module in itkBase.known_modules:
        itkBase.LoadModule(module, thisModule.__dict__)

# Regardless of how it was loaded, fill up the itk module with the ITK types
# and extras.
for k, v in itkTypes.__dict__.items():
    if k != 'itkCType' and not k.startswith('_'):
        setattr(thisModule, k, v)
for k, v in itkExtras.__dict__.items():
    if not k.startswith('_'):
        setattr(thisModule, k, v)

# Populate itk.ITKModuleName
for module, data in itkBase.module_data.items():
    attributes = {}
    _GetLazyAttributes(attributes)
    itkModule = itkLazy.LazyITKModule(module, attributes)
    setattr(thisModule, module, itkModule)

# Set the __path__ attribute, which is required for this module to be used as a
# package
setattr(thisModule, '__path__', __path__)

if itkConfig.LazyLoading:
    # this has to be the last step, else python gets confused about itkTypes
    # and itkExtras above. I'm not sure why...
    sys.modules[__name__] = thisModule
else:
    # do some cleanup
    del module, thisModule, itkModule
    del itkBase, itkConfig, itkLazy, itkTypes, itkExtras, os, sys



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

from __future__ import print_function

import sys
import commands
import tempfile
from optparse import OptionParser

parser = OptionParser(usage="usage: %prog")
parser.add_option(
    "--start-from",
    dest="startFrom",
    default=None,
    metavar="CLASS",
    help="")
parser.add_option(
    "-v",
    "--verbose",
    action="store_true",
    dest="verbose",
    default=False,
    help="")
parser.add_option(
    "-k",
    "--keep",
    action="store_true",
    dest="keep",
    default=False,
    help="")
(options, args) = parser.parse_args()

if options.keep:
    mode = "a"
else:
    mode = "w"
segfaultFile = file(args[0], mode)

logs = tempfile.NamedTemporaryFile()

lastClass = options.startFrom
ret = 1
while ret != 0:
    command = "python returnedTypeCoverage.py -v5 --exclude " + \
        sys.argv[1] + " --log-file '" + logs.name + "'"
    if lastClass:
        command += " --start-from " + lastClass
    if options.verbose:
        print(command)
    (ret, output) = commands.getstatusoutput(command)
    if ret != 0:
        # find last args (the ones which caused the segfault)
        faultyArgs = None
        logs.file.seek(0)
        for l in reversed(logs.file.readlines()):
            l = l.strip()
            if faultyArgs is None and l.startswith('('):
                faultyArgs = l
            if faultyArgs is not None:
                # find the last class
                if len(l) != 0 and l[0].isupper():
                    lastClass = l
                    break
        print(repr(faultyArgs))
        segfaultFile.write(faultyArgs + "\n")
        segfaultFile.flush()
